# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/tirons/src/akvo/akvo/gui/addFigure8Loop.ui'
#
# Created by: PyQt5 UI code generator 5.13.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_figure8LoopAdd(object):
    def setupUi(self, figure8LoopAdd):
        figure8LoopAdd.setObjectName("figure8LoopAdd")
        figure8LoopAdd.resize(400, 456)
        self.gridLayout = QtWidgets.QGridLayout(figure8LoopAdd)
        self.gridLayout.setObjectName("gridLayout")
        self.doubleSpinBox_3 = QtWidgets.QDoubleSpinBox(figure8LoopAdd)
        self.doubleSpinBox_3.setObjectName("doubleSpinBox_3")
        self.gridLayout.addWidget(self.doubleSpinBox_3, 12, 2, 1, 1)
        self.label_8 = QtWidgets.QLabel(figure8LoopAdd)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 7, 0, 1, 1)
        self.segments = QtWidgets.QSpinBox(figure8LoopAdd)
        self.segments.setMinimum(5)
        self.segments.setMaximum(200)
        self.segments.setProperty("value", 15)
        self.segments.setObjectName("segments")
        self.gridLayout.addWidget(self.segments, 14, 2, 1, 1)
        self.loopHeight = QtWidgets.QDoubleSpinBox(figure8LoopAdd)
        self.loopHeight.setDecimals(3)
        self.loopHeight.setMinimum(-99.0)
        self.loopHeight.setProperty("value", -0.001)
        self.loopHeight.setObjectName("loopHeight")
        self.gridLayout.addWidget(self.loopHeight, 10, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(figure8LoopAdd)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 13, 0, 1, 1)
        self.centreNorth2 = QtWidgets.QDoubleSpinBox(figure8LoopAdd)
        self.centreNorth2.setObjectName("centreNorth2")
        self.gridLayout.addWidget(self.centreNorth2, 6, 2, 1, 1)
        self.line_4 = QtWidgets.QFrame(figure8LoopAdd)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.gridLayout.addWidget(self.line_4, 8, 1, 1, 1)
        self.line = QtWidgets.QFrame(figure8LoopAdd)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line, 5, 0, 1, 1)
        self.centreEast1 = QtWidgets.QDoubleSpinBox(figure8LoopAdd)
        self.centreEast1.setMinimum(-99999999.0)
        self.centreEast1.setMaximum(99999999.99)
        self.centreEast1.setObjectName("centreEast1")
        self.gridLayout.addWidget(self.centreEast1, 1, 2, 1, 1)
        self.label_9 = QtWidgets.QLabel(figure8LoopAdd)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 12, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(figure8LoopAdd)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.line_3 = QtWidgets.QFrame(figure8LoopAdd)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.gridLayout.addWidget(self.line_3, 8, 2, 1, 1)
        self.label = QtWidgets.QLabel(figure8LoopAdd)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(figure8LoopAdd)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 6, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(figure8LoopAdd)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 14, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(figure8LoopAdd)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 17, 2, 1, 1)
        self.line_2 = QtWidgets.QFrame(figure8LoopAdd)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout.addWidget(self.line_2, 5, 2, 1, 1)
        self.centreEast2 = QtWidgets.QDoubleSpinBox(figure8LoopAdd)
        self.centreEast2.setObjectName("centreEast2")
        self.gridLayout.addWidget(self.centreEast2, 7, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(figure8LoopAdd)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 10, 0, 1, 1)
        self.centreNorth1 = QtWidgets.QDoubleSpinBox(figure8LoopAdd)
        self.centreNorth1.setMinimum(-999999999.0)
        self.centreNorth1.setMaximum(9999999999.0)
        self.centreNorth1.setObjectName("centreNorth1")
        self.gridLayout.addWidget(self.centreNorth1, 0, 2, 1, 1)
        self.loopTurns = QtWidgets.QSpinBox(figure8LoopAdd)
        self.loopTurns.setMinimum(1)
        self.loopTurns.setMaximum(100000)
        self.loopTurns.setObjectName("loopTurns")
        self.gridLayout.addWidget(self.loopTurns, 13, 2, 1, 1)
        self.line_5 = QtWidgets.QFrame(figure8LoopAdd)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.gridLayout.addWidget(self.line_5, 8, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(figure8LoopAdd)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 9, 0, 1, 1)
        self.loopRadius = QtWidgets.QDoubleSpinBox(figure8LoopAdd)
        self.loopRadius.setMinimum(0.1)
        self.loopRadius.setMaximum(600.0)
        self.loopRadius.setProperty("value", 50.0)
        self.loopRadius.setObjectName("loopRadius")
        self.gridLayout.addWidget(self.loopRadius, 9, 2, 1, 1)

        self.retranslateUi(figure8LoopAdd)
        self.buttonBox.accepted.connect(figure8LoopAdd.accept)
        self.buttonBox.rejected.connect(figure8LoopAdd.reject)
        QtCore.QMetaObject.connectSlotsByName(figure8LoopAdd)

    def retranslateUi(self, figure8LoopAdd):
        _translate = QtCore.QCoreApplication.translate
        figure8LoopAdd.setWindowTitle(_translate("figure8LoopAdd", "Dialog"))
        self.label_8.setText(_translate("figure8LoopAdd", "centre easting 2 (m)"))
        self.segments.setToolTip(_translate("figure8LoopAdd", "<html><head/><body><p>Currently Akvo/Merlin calculates circular loops using segments of wire, forming a polygon. Analytic circular loops may be added in the future. </p></body></html>"))
        self.loopHeight.setToolTip(_translate("figure8LoopAdd", "<html><head/><body><p>Akvo uses a positive down convention, a slight negative value to the loop height improves numerical stability using digital filtering hankel transforms. </p></body></html>"))
        self.label_6.setText(_translate("figure8LoopAdd", "turns"))
        self.label_9.setText(_translate("figure8LoopAdd", "TextLabel"))
        self.label_2.setText(_translate("figure8LoopAdd", "centre easting 1 (m)"))
        self.label.setText(_translate("figure8LoopAdd", "centre northing 1 (m)"))
        self.label_7.setText(_translate("figure8LoopAdd", "centre northing 2 (m)"))
        self.label_5.setText(_translate("figure8LoopAdd", "segments"))
        self.label_3.setText(_translate("figure8LoopAdd", "height (m)"))
        self.label_4.setText(_translate("figure8LoopAdd", "radius  (m)"))
        self.loopRadius.setToolTip(_translate("figure8LoopAdd", "<html><head/><body><p>Radius of the loop</p></body></html>"))
