# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/tirons/src/akvo/akvo/gui/addCircularLoop.ui'
#
# Created by: PyQt5 UI code generator 5.13.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_circularLoopAdd(object):
    def setupUi(self, circularLoopAdd):
        circularLoopAdd.setObjectName("circularLoopAdd")
        circularLoopAdd.resize(400, 438)
        self.gridLayout = QtWidgets.QGridLayout(circularLoopAdd)
        self.gridLayout.setObjectName("gridLayout")
        self.loopHeight = QtWidgets.QDoubleSpinBox(circularLoopAdd)
        self.loopHeight.setDecimals(3)
        self.loopHeight.setMinimum(-99.0)
        self.loopHeight.setProperty("value", -0.001)
        self.loopHeight.setObjectName("loopHeight")
        self.gridLayout.addWidget(self.loopHeight, 2, 2, 1, 1)
        self.centreEast = QtWidgets.QDoubleSpinBox(circularLoopAdd)
        self.centreEast.setMinimum(-99999999.0)
        self.centreEast.setMaximum(99999999.99)
        self.centreEast.setObjectName("centreEast")
        self.gridLayout.addWidget(self.centreEast, 1, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(circularLoopAdd)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.dip = QtWidgets.QDoubleSpinBox(circularLoopAdd)
        self.dip.setEnabled(False)
        self.dip.setMaximum(90.0)
        self.dip.setObjectName("dip")
        self.gridLayout.addWidget(self.dip, 7, 2, 1, 1)
        self.loopTurns = QtWidgets.QSpinBox(circularLoopAdd)
        self.loopTurns.setMinimum(1)
        self.loopTurns.setMaximum(100000)
        self.loopTurns.setObjectName("loopTurns")
        self.gridLayout.addWidget(self.loopTurns, 5, 2, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(circularLoopAdd)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 11, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(circularLoopAdd)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 5, 0, 1, 1)
        self.centreNorth = QtWidgets.QDoubleSpinBox(circularLoopAdd)
        self.centreNorth.setMinimum(-999999999.0)
        self.centreNorth.setMaximum(9999999999.0)
        self.centreNorth.setObjectName("centreNorth")
        self.gridLayout.addWidget(self.centreNorth, 0, 2, 1, 1)
        self.segments = QtWidgets.QSpinBox(circularLoopAdd)
        self.segments.setMinimum(5)
        self.segments.setMaximum(200)
        self.segments.setProperty("value", 15)
        self.segments.setObjectName("segments")
        self.gridLayout.addWidget(self.segments, 6, 2, 1, 1)
        self.loopRadius = QtWidgets.QDoubleSpinBox(circularLoopAdd)
        self.loopRadius.setMinimum(0.1)
        self.loopRadius.setMaximum(600.0)
        self.loopRadius.setProperty("value", 50.0)
        self.loopRadius.setObjectName("loopRadius")
        self.gridLayout.addWidget(self.loopRadius, 3, 2, 1, 1)
        self.label_7 = QtWidgets.QLabel(circularLoopAdd)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 7, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(circularLoopAdd)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 3, 0, 1, 1)
        self.label_8 = QtWidgets.QLabel(circularLoopAdd)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 8, 0, 1, 1)
        self.az = QtWidgets.QDoubleSpinBox(circularLoopAdd)
        self.az.setEnabled(False)
        self.az.setObjectName("az")
        self.gridLayout.addWidget(self.az, 8, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(circularLoopAdd)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.label = QtWidgets.QLabel(circularLoopAdd)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(circularLoopAdd)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 6, 0, 1, 1)
        self.label_9 = QtWidgets.QLabel(circularLoopAdd)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 4, 0, 1, 1)
        self.cwiseBox = QtWidgets.QComboBox(circularLoopAdd)
        self.cwiseBox.setObjectName("cwiseBox")
        self.cwiseBox.addItem("")
        self.cwiseBox.addItem("")
        self.gridLayout.addWidget(self.cwiseBox, 4, 2, 1, 1)

        self.retranslateUi(circularLoopAdd)
        self.buttonBox.accepted.connect(circularLoopAdd.accept)
        self.buttonBox.rejected.connect(circularLoopAdd.reject)
        QtCore.QMetaObject.connectSlotsByName(circularLoopAdd)

    def retranslateUi(self, circularLoopAdd):
        _translate = QtCore.QCoreApplication.translate
        circularLoopAdd.setWindowTitle(_translate("circularLoopAdd", "Dialog"))
        self.loopHeight.setToolTip(_translate("circularLoopAdd", "<html><head/><body><p>Akvo uses a positive down convention, a slight negative value to the loop height improves numerical stability using digital filtering hankel transforms. </p></body></html>"))
        self.label_3.setText(_translate("circularLoopAdd", "height (m)"))
        self.dip.setToolTip(_translate("circularLoopAdd", "<html><head/><body><p>The dip designates the angle between the normal and the ground. Zero designates a loop that is flat on the ground, whereas 90 is a vertical loop. This value in turn with azimuth allows for arbitrary orientation of the loops. If azimuth is zero, and dip is 90, the loop has a normal aligned in the northing direction. </p></body></html>"))
        self.label_6.setText(_translate("circularLoopAdd", "turns"))
        self.segments.setToolTip(_translate("circularLoopAdd", "<html><head/><body><p>Currently Akvo/Merlin calculates circular loops using segments of wire, forming a polygon. Analytic circular loops may be added in the future. </p></body></html>"))
        self.loopRadius.setToolTip(_translate("circularLoopAdd", "<html><head/><body><p>Radius of the loop</p></body></html>"))
        self.label_7.setText(_translate("circularLoopAdd", "dip (°)"))
        self.label_4.setText(_translate("circularLoopAdd", "radius (m)"))
        self.label_8.setText(_translate("circularLoopAdd", "azimuth (°)"))
        self.label_2.setText(_translate("circularLoopAdd", "centre easting (m)"))
        self.label.setText(_translate("circularLoopAdd", "centre northing (m)"))
        self.label_5.setText(_translate("circularLoopAdd", "segments"))
        self.label_9.setText(_translate("circularLoopAdd", "current flow"))
        self.cwiseBox.setItemText(0, _translate("circularLoopAdd", "clockwise"))
        self.cwiseBox.setItemText(1, _translate("circularLoopAdd", "anti-clockwise"))
